@@ -7,11 +7,11 @@
 ## http://search.cpan.org/dist/Devel-NYTProf/
 ##
 ###########################################################
-## $Id$
+## $Id: NYTProf.pm 1297 2010-06-10 10:24:38Z tim.bunce@gmail.com $
 ###########################################################
 package Devel::NYTProf;
 
-our $VERSION = '5.00'; # also change in Devel::NYTProf::Core
+our $VERSION = '4.01';
 
 package    # hide the package from the PAUSE indexer
     DB;
@@ -38,7 +38,7 @@ if ($use_db_sub) {                     # install DB::DB sub
 # DB::sub shouldn't be called, but needs to exist for perl <5.8.7 (<perl@24265)
 # Could be called in obscure cases, e.g. if "perl -d" (not -d:NYTProf)
 # was used with Devel::NYTProf loaded some other way
-sub sub { die "DB::sub called unexpectedly" }
+sub sub { die "DB::sub called unexpectly" }
 
 sub CLONE { DB::disable_profiler }
 
@@ -50,7 +50,7 @@ __END__
 
 =head1 NAME
 
-Devel::NYTProf - Powerful fast feature-rich Perl source code profiler
+Devel::NYTProf - Powerful fast feature-rich perl source code profiler
 
 =head1 SYNOPSIS
 
@@ -58,15 +58,11 @@ Devel::NYTProf - Powerful fast feature-rich Perl source code profiler
   perl -d:NYTProf some_perl.pl
 
   # convert database into a set of html files, e.g., ./nytprof/index.html
-  # and open a web browser on the nytprof/index.html file
-  nytprofhtml --open
+  nytprofhtml
 
   # or into comma separated files, e.g., ./nytprof/*.csv
   nytprofcsv
 
-A screencast about profiling perl code, including a detailed look at how to use
-NYTProf and how to optimize your code, is available at L<http://timbunce.blip.tv/file/3913278/>
-
 =head1 DESCRIPTION
 
 Devel::NYTProf is a powerful, fast, feature-rich perl source code profiler.
@@ -163,7 +159,8 @@ associated with the line of the source file that the previous statement starts o
 
 By default the statement profiler also determines the first line of the current
 block and the first line of the current statement, and accumulates times
-associated with those.
+associated with those. NYTProf is the only Perl profiler to perform block level
+profiling.
 
 Another innovation unique to NYTProf is automatic compensation for a problem
 inherent in simplistic statement-to-statement timing. Consider a statement that
@@ -204,8 +201,8 @@ For each subroutine called, separate counts and durations are stored I<for each
 location that called the subroutine>.
 
 Subroutine entry is detected by intercepting the C<entersub> opcode. Subroutine
-exit is detected via perl's internal save stack. As a result the subroutine
-profiler is both fast and robust.
+exit is detected via perl's internal save stack. The result is both extremely
+fast and very robust.
 
 =head3 Subroutine Recursion
 
@@ -224,21 +221,6 @@ The C<goto> will be shown with a very short time because it's effectively just
 a C<return>. The C<&destination> sub will show a call I<not> from the location
 of the C<goto> but from the location of the call to the sub that performed the C<goto>.
 
-=head3 accept()
-
-The perl built-in accept() function waits listening for a connection on a
-socket, and so is a key part of pure-perl network service applications.
-
-The time spent waiting for a remotely initiated connection can be relatively
-high but is not relevant to the performance of the application. So the accept()
-function is treated as a special case. The subroutine profiler discounts the
-time spent in the accept() function. It does this in a way that also discounts
-that time from all the callers up the call stack. The effect on the reports is
-that all accept() calls appear to be instant.
-
-The I<statement> profiler still shows the time actually spent in the statement
-that executed the accept() call.
-
 =head2 Application Profiling
 
 NYTProf records extra information in the data file to capture details that may
@@ -248,8 +230,7 @@ ranges of all the subroutines.
 NYTProf can profile applications that fork, and does so with no loss of
 performance.
 NYTProf detects the fork and starts writing a new profile file with the pid
-appended to the filename. Since L<nytprofhtml> only works with a single profile
-file you may want to merge multiple files using L<nytprofmerge>.
+appended to the filename.
 
 =head2 Fast Profiling
 
@@ -356,10 +337,6 @@ may be rewritten as
 so the profile won't show a statement count for line 2 in your source code
 because the C<return> was merged into the C<if> statement on the preceding line.
 
-Also 'empty' statements like C<1;> are removed entirely.  Such statements are
-empty because the optimizer has already removed the pointless constant in void
-context. It then goes on to remove the now empty statement (in perl >= 5.13.7).
-
 Using the C<optimize=0> option disables the optimizer so you'll get lower
 overall performance but more accurately assigned statement counts.
 
@@ -370,12 +347,13 @@ If you find any other examples of the effect of optimizer on NYTProf output
 
 Set to 0 to disable the collection of subroutine caller and timing details.
 
-=head2 blocks=1
+=head2 blocks=0
 
-Set to 1 to enable the determination of block and subroutine location per statement.
-This makes the profiler about 50% slower (as of July 2008) and produces larger
-output files, but you gain some valuable insight in where time is spent in the
-blocks within large subroutines and scripts.
+Set to 0 to disable the determination of block and subroutine location per statement.
+This makes the profiler about 50% faster (as of July 2008) and produces smaller
+output files, but you lose some valuable information. The extra cost is likely
+to be reduced in later versions anyway, as little optimization has been done on
+that part of the code.
 
 =head2 stmts=0
 
@@ -386,25 +364,6 @@ This significantly reduces the overhead of the profiler and can also be useful
 for profiling large applications that would normally generate a very large
 profile data file.
 
-=head2 calls=N
-
-This option is I<new and experimental>.
-
-With calls=1 (the default) subroutine call I<return> events are emitted into
-the data stream as they happen.  With calls=2 subroutine call I<entry> events
-are also emitted. With calls=0 no subroutine call events are produced.
-This option depends on the C<subs> option being enabled, which it is by default.
-
-The L<nytprofcalls> utility can be used to process this data. It too is I<new
-and experimental> and so likely to change.
-
-The subroutine profiler normally gathers data in memory and outputs a summary
-when the profile data is being finalized, usually when the program has finished.
-The summary contains aggregate information for all the calls from one location
-to another, but the details of individual calls have been lost.
-The calls option enables the recording of individual call events and thus
-more detailed analysis and reporting of that data.
-
 =head2 leave=0
 
 Set to 0 to disable the extra work done by the statement profiler
@@ -415,6 +374,9 @@ and returning into the middle of statement, or re-evaluating a loop condition.
 This feature also ensures that in embedded environments, such as mod_perl,
 the last statement executed doesn't accumulate the time spent 'outside perl'.
 
+NYTProf is the only line-level profiler to measure these times correctly.
+The profiler is fast enough that you shouldn't need to disable this feature.
+
 =head2 findcaller=1
 
 Force NYTProf to recalculate the name of the caller of the each sub instead of
@@ -478,11 +440,21 @@ likely to change in future.
 
 =head2 usecputime=1
 
-This option has been removed. Profiling won't be enabled if set.
+Measure user CPU + system CPU time instead of the real elapsed 'wall clock'
+time (which is the default).
 
-Use the L</clock=N> option to select a high-resolution CPU time clock, if
-available on your system, instead. That will give you higher resolution and work
-for the subroutine profiler as well.
+Measuring CPU time has the advantage of making the measurements independent of
+time spent blocked waiting for the cpu or network i/o etc. But it also has the
+severe disadvantage of having typically I<far> less accurate timings.
+
+Most systems use a 0.01 second granularity. With modern processors having multi-
+gigahertz clocks, 0.01 seconds is like a lifetime. The cpu time clock 'ticks'
+happen so rarely relative to the activity of a most applications that you'd
+have to run the code for many hours to have any hope of reasonably useful results.
+
+A better alternative would be to use the C<clock=N> option to select a
+high-resolution cpu time clock, if available on your system, because that'll
+give you higher resolution and work for the subroutine profiler as well.
 
 =head2 file=...
 
@@ -514,10 +486,9 @@ When perl exits normally it runs any code defined in C<END> blocks.
 NYTProf defines an END block that finishes profiling and writes out the final
 profile data.
 
-If the process ends due to a signal then END blocks are not executed so the
-profile will be incomplete and unusable.  The C<sigexit> option tells NYTProf
-to catch some signals (e.g. INT, HUP, PIPE, SEGV, BUS) and ensure a usable
-profile by executing:
+If the process ends due to a signal then END blocks are not executed.
+The C<sigexit> option tells NYTProf to catch some signals (e.g. INT, HUP, PIPE,
+SEGV, BUS) and ensure a usable by executing:
 
     DB::finish_profile();
     exit 1;
@@ -628,55 +599,6 @@ internals of perl mean that, in some cases, the information that's gathered is
 accurate but surprising. In some cases it can appear to be misleading.
 (Of course, in some cases it may actually be plain wrong. Caveat lector.)
 
-=head2 If Statement and Subroutine Timings Don't Match
-
-NYTProf has two profilers: a statement profiler that's invoked when perl moves
-from one perl statement to another, and a subroutine profiler that's invoked
-when perl calls or returns from a subroutine.
-
-The individual statement timings for a subroutine usually add up to slightly
-less than the exclusive time for the subroutine. That's because the handling of
-the subroutine call and return overheads is included in the exclusive time for
-the subroutine. The difference may only be a new microseconds but that may
-become noticeable for subroutines that are called hundreds of thousands of times.
-
-The statement profiler keeps track how much time was spent on overheads, like
-writing statement profile data to disk. The subroutine profiler subtracts the
-overheads that have accumulated between entering and leaving the subroutine in
-order to give a more accurate profile.  The statement profiler is generally
-very fast because most writes get buffered for zip compression so the profiler
-overhead per statement tends to be very small, often a single 'tick'.
-The result is that the accumulated overhead is quite noisy. This becomes more
-significant for subroutines that are called frequently and are also fast.
-This may be another, smaller, contribution to the discrepancy between statement
-time and exclusive times.
-
-=head2 If Headline Subroutine Timings Don't Match the Called Subs
-
-Overall subroutine times are reported with a headline like C<spent 10s (2+8) within ...>.
-In this example, 10 seconds were spent inside the subroutine (the "inclusive
-time") and, of that, 8 seconds were spent in subroutines called by this one.
-That leaves 2 seconds as the time spent in the subroutine code itself (the
-"exclusive time", sometimes also called the "self time").
-
-The report shows the source code of the subroutine. Lines that make calls to
-other subroutines are annotated with details of the time spent in those calls.
-
-Sometimes the sum of the times for calls made by the lines of code in the
-subroutine is less than the inclusive-exclusive time reported in the headline
-(10-2 = 8 seconds in the example above).
-
-What's happening here is that calls to other subroutines are being made but
-NYTProf isn't able to determine the calling location correctly so the calls
-don't appear in the report in the correct place.
-
-Using an old version of perl is one cause (see below). Another is calling
-subroutines that exit via C<goto &sub;> - most frequently encountered in
-AUTOLOAD subs and code using the L<Memoize> module.
-
-In general the overall subroutine timing is accurate and should be trusted more
-than the sum of statement or nested sub call timings.
-
 =head2 Perl 5.10.1+ (or else 5.8.9+) is Recommended
 
 These versions of perl yield much more detailed information about calls to
@@ -786,21 +708,17 @@ You can reduce the cost of profiling by adjusting some options. The trade-off
 is reduced detail and/or accuracy in reports.
 
 If you don't need statement-level profiling then you can disable it via L</stmts=0>.
-To further boost statement-level profiling performance try L</leave=0> but note that
-I<will> apportion timings for some kinds of statements less accurate).
+If you do need it but don't mind loosing block-level timings then set L</blocks=0>.
+If you want need still more speed then set L</leave=0> to disable the
+adjustments made for statements that 'leave' the current control flow (doing
+this I<will> make timings for some kinds of statements less accurate).
 
-If you don't need calls stacks or flamegrph then disable it via L</calls=0>.
 If you don't need subroutine profiling then you can disable it via L</subs=0>.
 If you do need it but don't need timings for perl opcodes then set L</slowops=0>.
 
-Generally speaking, setting calls=0 and slowops=0 will give you a useful boost
-with the least loss of detail.
-
 Another approach is to only enable NYTProf in the sections of code that
 interest you. See L</RUN-TIME CONTROL OF PROFILING> for more details.
 
-To speed up L<nytprofhtml> try using the --minimal (-m) or --no-flame options.
-
 =head1 REPORTS
 
 The L<Devel::NYTProf::Data> module provides a low-level interface for loading
@@ -813,20 +731,19 @@ perl. (Though the module is in a state of flux and may be deprecated soon.)
 Included in the bin directory of this distribution are some scripts which
 turn the raw profile data into more useful formats:
 
-=head2 nytprofhtml
+=head2 nytprofcsv
 
-Creates attractive, richly annotated, and fully cross-linked html
-reports (including statistics, source code and color highlighting).
-This is the main report generation tool for NYTProf.
+Creates comma delimited profile reports. Old and limited.
 
 =head2 nytprofcg
 
 Translates a profile into a format that can be loaded into KCachegrind
 L<http://kcachegrind.sourceforge.net>
 
-=head2 nytprofcalls
+=head2 nytprofhtml
 
-Reads a profile and processes the calls events it contains.
+Creates attractive, richly annotated, and fully cross-linked html
+reports (including statistics, source code and color highlighting).
 
 =head2 nytprofmerge
 
@@ -842,12 +759,6 @@ touch with us. Meanwhile, profiling is disabled when a thread is created, and
 NYTProf tries to ignore any activity from perl interpreters other than the
 first one that loaded it.
 
-=head2 Coro
-
-The C<Devel::NYTProf> subroutine profiler gets confused by the stack gymnastics
-performed by the L<Coro> module and aborts. When profiling applications that
-use Coro you should disable the subroutine profiler using the L</subs=0> option.
-
 =head2 For perl < 5.8.8 it may change what caller() returns
 
 For example, the L<Readonly> module croaks with "Invalid tie" when profiled with
@@ -916,25 +827,27 @@ helps reduce the cumulative error that naturally occurs when adding together
 many timings. When using these clocks NYTProf outputs timings as a count of 100
 nanosecond ticks.
 
-=head3 CLOCK_MONOTONIC
-
-CLOCK_MONOTONIC represents the amount of time since an unspecified point in
-the past (typically system start-up time).  It increments uniformly
-independent of adjustments to 'wallclock time'. NYTProf will use this clock by
-default, if available.
-
 =head3 CLOCK_REALTIME
 
 CLOCK_REALTIME is typically the system's main high resolution 'wall clock time'
 source.  The same source as used for the gettimeofday() call used by most kinds
 of perl benchmarking and profiling tools.
 
+If your system doesn't support clock_gettime() then NYTProf will use
+gettimeofday(), or the nearest equivalent,
+
 The problem with real time is that it's far from simple. It tends to drift and
 then be reset to match 'reality', either sharply or by small adjustments (via the
 adjtime() system call).
 
 Surprisingly, it can also go backwards, for reasons explained in
-http://preview.tinyurl.com/5wawnn so CLOCK_MONOTONIC is preferred.
+http://preview.tinyurl.com/5wawnn
+
+=head3 CLOCK_MONOTONIC
+
+CLOCK_MONOTONIC represents the amount of time since an unspecified point in
+the past (typically system start-up time).  It increments uniformly
+independent of adjustments to 'wallclock time'.
 
 =head3 CLOCK_VIRTUAL
 
@@ -968,9 +881,6 @@ A future version of NYTProf should be able to list the supported clocks.
 
 This section lists other clock sources that NYTProf may use.
 
-If your system doesn't support clock_gettime() then NYTProf will use
-gettimeofday(), or the nearest equivalent,
-
 =head3 gettimeofday
 
 This is the traditional high resolution time of day interface for most
@@ -999,10 +909,6 @@ Why 'realtime' can appear to go backwards:
 
   http://preview.tinyurl.com/5wawnn
 
-The PostgreSQL pg_test_timing utility documentation has a good summary of timing issues:
-
-  http://www.postgresql.org/docs/9.2/static/pgtesttiming.html
-
 =for comment
 http://preview.tinyurl.com/5wawnn redirects to:
 http://groups.google.com/group/comp.os.linux.development.apps/tree/browse_frm/thread/dc29071f2417f75f/ac44671fdb35f6db?rnum=1&_done=%2Fgroup%2Fcomp.os.linux.development.apps%2Fbrowse_frm%2Fthread%2Fdc29071f2417f75f%2Fc46264dba0863463%3Flnk%3Dst%26rnum%3D1%26
@@ -1084,6 +990,10 @@ I<cpu-time> clock instead of a real-time one. That should be much less
 noisy, though you will lose visibility of wait-times due to network
 and disk I/O, for example.
 
+If your system doesn't support the C<clock=N> option then you could try
+using the C<usecputime=1> option. That will give you cpu-time measurements
+but only at a very low 1/100th of a second resolution.
+
 =head1 BUGS
 
 Possibly. All complex software has bugs. Let me know if you find one.
@@ -1116,7 +1026,7 @@ file as a stream of chunks of data.
 
 =head1 TROUBLESHOOTING
 
-=head2 "Profile data incomplete, ..." or "File format error: ..."
+=head2 "Profile data incomplete, ..."
 
 This error message means the file doesn't contain all the expected data.
 That may be because it was truncated (perhaps the filesystem was full) or,
@@ -1134,59 +1044,13 @@ If the sudden death was due to a signal then L</sigexit=1> may help.
 If the sudden death was due to calling C<POSIX::_exit($status)> then you'll
 need to call L</finish_profile> before calling C<POSIX::_exit>.
 
-You'll also get this error if the code trying to read the profile is itself
-being profiled. That's most likely to happen if you enable profiling via the
-C<PERL5OPT> environment variable and have forgotten to unset it.
+=head2 Some files don't have profile information
 
-=head2 Some source files don't have profile information
-
-This is usually due to NYTProf being initialized after some perl files have
-already been compiled.
+This is usually due to NYTProf being loaded after the other files, for example
 
 If you can't alter the command line to add "C<-d:NYTProf>" you could try using
 the C<PERL5OPT> environment variable. See L</PROFILING>.
 
-You could also try using the L</use_db_sub=1> option.
-
-=head2 Eval ... has unknown invoking fid
-
-When using the statement profiler you may see a warning message like this:
-
-  Eval '(eval 2)' (fid 9, flags:viastmt,savesrc) has unknown invoking fid 10
- 
-Notice that the eval file id (fid 9 in this case) is lower than the file id
-that invoked the eval (fid 10 in this case). This is a known problem caused by
-the way perl works and how the profiler assigns and outputs the file ids.
-The invoking fid is known but gets assigned a fid and output after the fid for
-the eval, and that causes the warning when the file is read.
-
-=head2 Warning: %d subroutine calls had negative time
-
-There are two likely causes for this: clock instability, or accumulated timing
-errors.
-
-Clock instability, if present on your system, is most likely to be noticeable on
-very small/fast subroutines that are called very few times.
-
-Accumulated timing errors can arise because the subroutine profiler uses
-floating point values (NVs) to store the times.  They are most likely to be
-noticed on subroutines that are called a few times but which make a large
-number of calls to very fast subroutines (such as opcodes). In this case the
-accumulated time apparently spent making those calls can be greater than the
-time spent in the calling subroutine.
-
-If you rerun nytprofhtml (etc.) with the L</trace=N> option set >0 you'll see
-trace messages like  "%s call has negative time: incl %fs, excl %fs" for each
-affected subroutine.
-
-Try profiling with the L</slowops=N> option set to 0 to disable the profiling
-of opcodes. Since opcodes often execute in a few microseconds they are a common
-cause of this warning.
-
-You could also try recompiling perl to use 'long doubles' for the NV floating
-point type (use Configure -Duselongdouble). If you try this please let me know.
-I'd also happily take a patch to use long doubles, if available, by default.
-
 =head1 AUTHORS AND CONTRIBUTORS
 
 B<Tim Bunce> (L<http://www.tim.bunce.name> and L<http://blog.timbunce.org>)
@@ -1249,7 +1113,7 @@ Statement-level profilers:
 Devel::NYTProf is a (now distant) fork of Devel::FastProf, which was itself an
 evolution of Devel::SmallProf.
 
-Adam Kaplan forked Devel::FastProf and added html report generation (based on
+Adam Kaplan took Devel::FastProf and added html report generation (based on
 Devel::Cover) and a test suite - a tricky thing to do for a profiler.
 Meanwhile Tim Bunce had been extending Devel::FastProf to add novel
 per-sub and per-block timing, plus subroutine caller tracking.
@@ -1258,7 +1122,7 @@ When Devel::NYTProf was released Tim switched to working on Devel::NYTProf
 because the html report would be a good way to show the extra profile data, and
 the test suite made development much easier and safer.
 
-Then Tim went a little crazy and added a slew of new features, in addition to
+Then he went a little crazy and added a slew of new features, in addition to
 per-sub and per-block timing and subroutine caller tracking. These included the
 'opcode interception' method of profiling, ultra-fast and robust inclusive
 subroutine timing, doubling performance, plus major changes to html reporting
@@ -1270,14 +1134,7 @@ keep NYTProf working with the latest development perl versions. Nicholas Clark
 added zip compression, many optimizations, and C<nytprofmerge>.
 Jan Dubois contributed Windows support.
 
-Adam's work was sponsored by The New York Times Co. L<http://open.nytimes.com>.
-Tim's work was partly sponsored by Shopzilla L<http://www.shopzilla.com> during 2008
-but hasn't been sponsored since then.
-
-For the record, Tim has never worked for the New York Times nor has he received any
-kind of sponsorship or support from them in relation to NYTProf. The name of
-this module is simply result of the history outlined above, which can be
-summarised as: Adam forked an existing module when he added his enhancements
-and Tim didn't.
+Adam's work is sponsored by The New York Times Co. L<http://open.nytimes.com>.
+Tim's work was partly sponsored by Shopzilla L<http://www.shopzilla.com> during 2008.
 
 =cut
