@@ -7,20 +7,17 @@
 # http://search.cpan.org/dist/Devel-NYTProf/
 #
 ###########################################################
-# $Id$
+# $Id: Core.pm 1297 2010-06-10 10:24:38Z tim.bunce@gmail.com $
 ###########################################################
 package Devel::NYTProf::Core;
 
 
 use XSLoader;
 
-our $VERSION = '5.00';    # increment with XS changes too
+our $VERSION = '4.01';    # increment with XS changes too
 
 XSLoader::load('Devel::NYTProf', $VERSION);
 
-# Fudging for https://rt.cpan.org/Ticket/Display.html?id=82256
-$Devel::NYTProf::StrEvalTestPad = ($] <= 5.017004) ? ";\n" : "";
-
 if (my $NYTPROF = $ENV{NYTPROF}) {
     for my $optval ( $NYTPROF =~ /((?:[^\\:]+|\\.)+)/g) {
         my ($opt, $val) = $optval =~ /^((?:[^\\=]+|\\.)+)=((?:[^\\=]+|\\.)+)\z/;
